#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : Python.
# @File         : simbert
# @Time         : 2020-04-09 14:08
# @Author       : yuanjie
# @Email        : yuanjie@xiaomi.com
# @Software     : PyCharm
# @Description  : 


import numpy as np
from collections import Counter
from bert4keras.backend import keras, K
from bert4keras.models import build_transformer_model
from bert4keras.tokenizers import Tokenizer
from bert4keras.snippets import sequence_padding, AutoRegressiveDecoder
from bert4keras.snippets import uniout
from keras.layers import *


def normalize(x):
    return x / np.linalg.norm(x, 2, axis=len(x.shape) > 1, keepdims=True)


# bert配置
config_path = './chinese_simbert_L-12_H-768_A-12/bert_config.json'
checkpoint_path = './chinese_simbert_L-12_H-768_A-12/bert_model.ckpt'
dict_path = './chinese_simbert_L-12_H-768_A-12/vocab.txt'

# 建立分词器
tokenizer = Tokenizer(dict_path, do_lower_case=True)

# 建立加载模型
bert = build_transformer_model(
    config_path,
    checkpoint_path,
    with_pool='linear',
    application='unilm',
    return_keras_model=True  # True: bert.predict([np.array([token_ids]), np.array([segment_ids])])
)

bert_ = build_transformer_model(
    config_path,
    checkpoint_path,
    with_pool='linear',
    application='unilm',
    return_keras_model=False  # True: bert.predict([np.array([token_ids]), np.array([segment_ids])])
)

encoder = keras.models.Model(bert_.model.inputs, bert_.model.outputs[0])


# seq2seq = keras.models.Model(bert.model.inputs, bert.model.outputs[1])

def sentence_encode(texts: list, batch_size=64):
    X, S = [], []
    for t in texts:
        x, s = tokenizer.encode(t)
        X.append(x)
        S.append(s)
    X = sequence_padding(X)
    S = sequence_padding(S)

    vec = encoder.predict([X, S], batch_size)
    return normalize(vec).tolist()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : mi.
# @File         : video
# @Time         : 2020/8/31 4:26 下午
# @Author       : yuanjie
# @Email        : yuanjie@xiaomi.com
# @Software     : PyCharm
# @Description  : 

from tql.pipe import *
from moviepy.editor import *


# TODO: 抽帧去重
def video2picture(video='/fds/1_Work/2_DownVideo/videos/互联网_yidian_V_07d0oZik.mp4'):
    p = Path(video)
    pic_dir = ''.join(p.name[::-1].split('.')[-1:])[::-1]
    (p.parent / pic_dir).mkdir(exist_ok=True)

    with VideoFileClip(video) as clip:
        duration = int(clip.duration)
        for i in tqdm(range(duration)):
            clip_ = clip.subclip(i, i + 1)
            clip_.save_frame(f'{i}.png')


def video2audio(path_pair, verbose=False, subclip=None, ffmpeg_params=["-f", "mp3"]):
    """
        clip = VideoFileClip('蛋清和蛋黄是这么分离的.720p').subclip(3, 7)

    :param paths: (video_path, audio_path)
    :param subclip:
    :param ffmpeg_params:
    :return:
    """
    video_path, audio_path = path_pair

    with VideoFileClip(video_path) as clip:
        duration = int(clip.duration)
        if subclip:
            s, e = subclip[0], duration if subclip is None or duration < subclip[1] else subclip[1]
            clip = clip.subclip(s, e)

        clip.audio.write_audiofile(
            audio_path, fps=None, nbytes=2, buffersize=2000,
            codec=None, bitrate=None, ffmpeg_params=ffmpeg_params,
            write_logfile=False, verbose=verbose, logger='bar' if verbose else None
        )


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : mi.
# @File         : load_yaml
# @Time         : 2020-03-21 21:40
# @Author       : yuanjie
# @Email        : yuanjie@xiaomi.com
# @Software     : PyCharm
# @Description  : 


import yaml

_ = yaml.safe_load(open("cfg.yaml").read())
print(_)
print(eval(_['email']['biz']['msg_fn'])(777777777))


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : mi.
# @File         : get_raw_url
# @Time         : 2020/9/29 11:31 上午
# @Author       : yuanjie
# @Email        : yuanjie@xiaomi.com
# @Software     : PyCharm
# @Description  :
import requests


def get_raw_url(cpDocId, cp='qiehao'):
    json = {'cp': cp, 'url': '', 'cpDocId': cpDocId}
    r = requests.post('http://10.142.116.14:8235/v1/fdl/geturl', json=json)
    return r.text


if __name__ == '__main__':
    docid = 'RZBLab6kOtm0T6LiAXCV07rt6DlGoh41NdgFJsCkBdjlYzQZ3P2QsA2IeI9cX5GqhgzSml9I3t8v8BPKgvA6tz5YUNWXvQuBm42'
    print(get_raw_url(docid))


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : tql-App.
# @File         : helloworld
# @Time         : 2020-01-09 10:38
# @Author       : yuanjie
# @Email        : yuanjie@xiaomi.com
# @Software     : PyCharm
# @Description  : 
from typing import Optional
from fastapi import FastAPI, Form, Depends
from pydantic import BaseModel
from starlette.staticfiles import StaticFiles
from starlette.requests import Request
from starlette.responses import RedirectResponse
from starlette.status import *

app = FastAPI(debug=True)


# app = FastAPI(
#     openapi_url="/poetry/openapi.json",
#     docs_url="/poetry/docs",
#     redoc_url="/poetry/redoc",
#     swagger_ui_oauth2_redirect_url="/poetry/docs/oauth2-redirect"
# )


class Item(BaseModel):
    name: str
    price: float
    is_offer: bool = None


@app.get("/")
def read_root():
    return {"Hello": "World"}


async def parse_my_form(request: Request) -> dict:
    form_data = await request.form()
    return dict(form_data)


@app.post("/my-form-endpoint")
def my_endpoint(form_data: dict = Depends(parse_my_form)):

    print(form_data)
    return "post succeed"


@app.get("/items/{item_id}")
def read_item(item_id: int, q: str = None):
    return {"item_id": item_id, "q": q}


@app.put("/items/{item_id}")
async def create_item(item_id: int,
                      item: Item,
                      q: str = None):
    result = {"item_id": item_id, **item.dict()}
    if q:
        result.update({"q": q})
    return result


# @app.put("/data/{data}")
# def update_item(item_id: int, item: Item):
#     return {"item_name": item.name, "item_id": item_id}


if __name__ == '__main__':
    import os
    import socket

    me = socket.gethostname() == 'yuanjie-Mac.local'

    uvicorn = "uvicorn" if me else "/opt/soft/python3/bin/uvicorn"

    main_file = __file__.split('/')[-1].split('.')[0]

    os.system(f"uvicorn {main_file}:app --reload --host 0.0.0.0")

    # os.system(f"gunicorn -c gun.py -w 1 -k uvicorn.workers.UvicornWorker {main_file}:app")



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : mi.
# @File         : mp3
# @Time         : 2020/9/1 10:42 上午
# @Author       : yuanjie
# @Email        : yuanjie@xiaomi.com
# @Software     : PyCharm
# @Description  : 


import time
from datetime import datetime
import pandas as pd
from typing import Optional
from fastapi import FastAPI, Form, Depends, File, UploadFile
from pydantic import BaseModel
from starlette.staticfiles import StaticFiles
from starlette.requests import Request
from starlette.responses import \
    RedirectResponse, FileResponse, HTMLResponse, PlainTextResponse, StreamingResponse
from starlette.status import *

ROUTE = "/mongo"
app = FastAPI(
    debug=True,
    openapi_url=f"{ROUTE}/openapi.json",
    docs_url=f"{ROUTE}/docs",
    redoc_url=f"{ROUTE}/redoc",
    swagger_ui_oauth2_redirect_url=f"{ROUTE}/docs/oauth2-redirect"
)


@app.get("/mp3/{file}")
def read_root(file):
    path = f"/Users/yuanjie/Desktop/{file}"

    # file_like = open(path, mode="rb")
    # return StreamingResponse(file_like, media_type="video/mp4")

    return FileResponse(path, media_type="video/mp4")


if __name__ == '__main__':
    import socket
    import uvicorn

    me = socket.gethostname() == 'yuanjie-Mac.local'
    main_file = __file__.split('/')[-1].split('.')[0]

    # app参数必须是个字符串才能开启热更新
    uvicorn.run(
        app=f"{main_file}:app", host='0.0.0.0',
        port=8000, workers=1,
        reload=True,
        debug=True
    )



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : Python.
# @File         : __init__.py
# @Time         : 2020-03-20 11:25
# @Author       : yuanjie
# @Email        : yuanjie@xiaomi.com
# @Software     : PyCharm
# @Description  : 


from .Mongo import Mongo
from .Mysql import Mysql
from .Redis import Redis
from .Neo4j import Neo4j
from .ANN import ANN



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : mi.
# @File         : ocr_app
# @Time         : 2020/9/9 10:49 上午
# @Author       : yuanjie
# @Email        : yuanjie@xiaomi.com
# @Software     : PyCharm
# @Description  : 


import time
from datetime import datetime
import pandas as pd
from typing import Optional
from fastapi import FastAPI, Form, Depends, File, UploadFile
from pydantic import BaseModel
from starlette.staticfiles import StaticFiles
from starlette.requests import Request
from starlette.responses import \
    RedirectResponse, FileResponse, HTMLResponse, PlainTextResponse
from starlette.status import *

ROUTE = "/ocr"
app = FastAPI(
    debug=True,
    openapi_url=f"{ROUTE}/openapi.json",
    docs_url=f"{ROUTE}/docs",
    redoc_url=f"{ROUTE}/redoc",
    swagger_ui_oauth2_redirect_url=f"{ROUTE}/docs/oauth2-redirect"
)

import base64
import requests
import pytesseract
from PIL import Image
from traceback import format_exc  # https://www.cnblogs.com/klchang/p/4635040.html
from mi.sdk import BaiduApi
from tempfile import TemporaryFile

baidu_api = BaiduApi()


@app.post("/ocr/{biz}")
async def ocr(request: Request, kwargs: dict, biz):
    image = kwargs['image']

    options = kwargs.get('options')
    lang = kwargs.get('lang', 'chi_sim')

    try:

        if biz == 'baidu':
            if len(image) < 1024:
                image = requests.get(image).content
                baidu_api

            else:
                image = base64.b64decode(image)
            return baidu_api.ocr_basic_general(image, options)

        elif biz == 'google':
            if len(image) < 1024:
                image = requests.get(image, stream=True).raw
                image = Image.open(image)

                return pytesseract.image_to_string(image, lang=lang)
            else:
                image = base64.b64decode(image)

                with TemporaryFile() as f:
                    # Read/write to the file
                    f.write(image)
                    # Seek back to beginning and read the data
                    f.seek(0)
                    # data = f.read()
                    image = Image.open(f)

                    return pytesseract.image_to_string(image, lang=lang)

    except Exception as e:
        return {'error': format_exc()}


if __name__ == '__main__':
    import socket
    import uvicorn

    me = socket.gethostname() == 'yuanjie-Mac.local'
    main_file = __file__.split('/')[-1].split('.')[0]

    # app参数必须是个字符串才能开启热更新
    uvicorn.run(app=f"{main_file}:app", host='0.0.0.0', port=8000, workers=1, debug=True, reload=True)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : mi.
# @File         : send_email
# @Time         : 2020-03-04 13:58
# @Author       : yuanjie
# @Email        : yuanjie@xiaomi.com
# @Software     : PyCharm
# @Description  : 

""":arg
https://cloud.mioffice.cn/#/mail?_k=5lkm15
https://t.mioffice.cn/mail/#/
"""

import smtplib
from email.mime.text import MIMEText
from email.header import Header
from ..utils import isMac

from datetime import datetime


def send_email(subject="",
               jobid="jobid",
               msg="",
               receivers='yuanjie@xiaomi.com',
               isstaging=None,
               _subtype='html',
               msg_prefix='👍',
               msg_suffix='👍',
               msg_fn=lambda x: x,
               **kwargs):
    """

    :param subject:
    :param jobid:
    :param msg:
    :param receivers:
    :param isstaging:
    :param _subtype:
    :param msg_prefix:
    :param msg_suffix:
    :param msg_fn:
    :param kwargs:
    :return:
    """

    # process
    subject = f"👉{subject}😏{jobid}📅{datetime.now().__str__()[:10]}"
    msg = f"{msg_prefix}{msg_fn(msg)}{msg_suffix}"

    if isinstance(receivers, str) and receivers.__contains__("@"):
        receivers = [receivers]

    token = {
        "mail.b2c.srv": "U92BzW2jqR@xiaomi.com",
        "mail.test.b2c.srv": "134a1ab8c0efbe14884b9956321818e0@xiaomi.com"
        # "9297367afe24f39009dae012d2fd0342@xiaomi.com"
    }
    isstaging = 1 if isMac else isstaging
    smtp = smtplib.SMTP("mail.test.b2c.srv" if isstaging else "mail.b2c.srv", 25)
    sender = token[smtp._host]

    message = MIMEText(msg, _subtype, 'utf-8')
    message['Subject'] = Header(subject, 'utf-8')
    message['From'] = sender
    message['To'] = ",".join(receivers)

    try:
        smtp.sendmail(sender, receivers, message.as_string())
        print("邮件发送成功")
    except smtplib.SMTPException as e:
        print(f"{e}: 无法发送邮件")


if __name__ == '__main__':
    send_email(isstaging=1)
